@@ -45,7 +45,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #define DEBUG_VAR(fmt,var,args...)
 #endif
 
-#define ANIM_FRAMES 8
 #define ANIM_TIME   500
 
 /*----------------------------------------------------------------------------*/
@@ -78,6 +77,7 @@ static void show_help (GtkWidget *, ConnectPlugin *);
 static void show_menu (ConnectPlugin *);
 static void update_icon (ConnectPlugin *);
 static gboolean animate (ConnectPlugin *c);
+static void cache_animation (ConnectPlugin *c, gboolean clear);
 static void connect_button_press_event (GtkButton *, ConnectPlugin *);
 
 /*----------------------------------------------------------------------------*/
@@ -389,20 +389,41 @@ static void update_icon (ConnectPlugin *c)
 
 static gboolean animate (ConnectPlugin *c)
 {
-    char *icon;
-
-    if (c->vnc_sess_count + c->ssh_sess_count > 0)
+    if (c->enabled && c->signed_in && c->vnc_sess_count + c->ssh_sess_count > 0)
     {
-        c->anim_frame++;
-        if (c->anim_frame > (ANIM_FRAMES - 1) || c->animate == FALSE) c->anim_frame = 0;
-
-        icon = g_strdup_printf ("rpc-active%d", c->anim_frame);
-        set_taskbar_icon (c->tray_icon, icon, get_icon_size ());
-        g_free (icon);
+        if (c->animate)
+        {
+            c->anim_frame++;
+            if (c->anim_frame > (ANIM_FRAMES - 1)) c->anim_frame = 0;
+            gtk_image_set_from_pixbuf (GTK_IMAGE (c->tray_icon), c->anim[c->anim_frame]);
+        }
+        else
+        {
+            if (c->anim_frame != -1)
+            {
+                gtk_image_set_from_pixbuf (GTK_IMAGE (c->tray_icon), c->anim[0]);
+                c->anim_frame = -1;
+            }
+        }
     }
     return TRUE;
 }
 
+static void cache_animation (ConnectPlugin *c, gboolean clear)
+{
+    char *iname;
+    int count;
+
+    for (count = 0; count < ANIM_FRAMES; count++)
+    {
+        if (clear && c->anim[count]) g_object_unref (c->anim[count]);
+        iname = g_strdup_printf ("rpc-active%d", count);
+        c->anim[count] = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), iname,
+            get_icon_size (), GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+        g_free (iname);
+    }
+}
+
 /*----------------------------------------------------------------------------*/
 /* wf-panel plugin functions                                                  */
 /*----------------------------------------------------------------------------*/
@@ -418,6 +439,7 @@ static void connect_button_press_event (GtkButton *, ConnectPlugin *c)
 /* Handler for system config changed message from panel */
 void connect_update_display (ConnectPlugin *c)
 {
+    cache_animation (c, TRUE);
     update_icon (c);
 }
 
@@ -464,6 +486,9 @@ void connect_init (ConnectPlugin *c)
     c->enabled = FALSE;
     c->enabling = FALSE;
 
+    /* Cache animation */
+    cache_animation (c, FALSE);
+
     /* Set up callbacks to see if Connect is on DBus */
     c->watch = g_bus_watch_name (G_BUS_TYPE_SESSION, "com.raspberrypi.Connect", 0,
         (GBusNameAppearedCallback) cb_name_owned, (GBusNameVanishedCallback) cb_name_unowned, c, NULL);
@@ -473,10 +498,15 @@ void connect_init (ConnectPlugin *c)
 
 void connect_destructor (ConnectPlugin *c)
 {
+    int count;
+
     g_bus_unwatch_name (c->watch);
 
     if (c->icon_timer) g_source_remove (c->icon_timer);
 
+    for (count = 0; count < ANIM_FRAMES; count++)
+        if (c->anim[count]) g_object_unref (c->anim[count]);
+
     g_free (c);
 }
 
